import argparse
import os
import re
from typing import Optional


from browser_hub.processes import is_profile_active
from browser_hub.config import (
    Config,
    Profile,
    load_config_or_exit,
)


class BrowserHub:
    open_url: str
    config: Config

    def __init__(self, open_url: str, config: Config) -> None:
        self.open_url = open_url
        self.config = config

    def find_matched_profile(self) -> Optional[Profile]:
        for profile in self.config.profiles:
            matches = any(p in self.open_url for p in profile.url_patterns)
            if matches:
                return profile
        return None

    def should_open_last_active_profile(self) -> bool:
        for url in self.config.profile_specific_urls:
            if url in self.open_url:
                return True
        return False

    def transform_url(self, profile: Profile, url: str) -> str:
        for transformer in profile.url_transformers:
            keywords_matches = any(k for k in transformer.keywords if k in url)
            if keywords_matches:
                transformed_url = re.sub(
                    transformer.from_url_regex,
                    transformer.to_url,
                    url,
                )
                return transformed_url
        return url

    def find_active_profile(self) -> Optional[Profile]:
        for profile in self.config.profiles:
            active = is_profile_active(
                profile.browser.process_names,
                profile.browser.cmd_includes_regex,
                profile.browser.cmd_excludes_regex,
            )
            if active:
                return profile
        return None

    def open(self) -> None:
        profile = self.find_matched_profile()
        if not profile and self.should_open_last_active_profile():
            # Find the active profile based on the running process
            profile = self.find_active_profile()

        if profile:
            url = self.transform_url(profile, self.open_url)
            os.system(profile.browser.open_cmd.format(url=url))
        else:
            os.system(self.config.default_browser_open_cmd.format(url=self.open_url))


def cli() -> None:
    parser = argparse.ArgumentParser(
        prog="browser-hub",
        description="Open the right browser profile",
    )
    parser.add_argument("url")
    args = parser.parse_args()

    config = load_config_or_exit()

    BrowserHub(open_url=args.url, config=config).open()


if __name__ == "__main__":
    cli()



import json
from os import path
from sys import stderr
from typing import List, Optional
from pydantic import BaseModel, Field, ValidationError

CONFIG_FILE_PATH = path.expanduser("~/.config/browser-hub/config.json")


class URLTransformer(BaseModel):
    keywords: List[str] = Field(min_length=1)
    from_url_regex: str
    to_url: str


class Browser(BaseModel):
    open_cmd: str
    process_names: List[str] = Field(min_length=1)
    cmd_includes_regex: str
    cmd_excludes_regex: Optional[str]


class Profile(BaseModel):
    name: str
    browser: Browser
    url_patterns: List[str] = Field(min_length=1)
    url_transformers: List[URLTransformer] = Field(default_factory=list)


class Config(BaseModel):
    default_browser_open_cmd: str
    profiles: List[Profile] = Field(min_length=1)
    # Open these URLs in the profile-specific browser if one is open
    profile_specific_urls: List[str]


def _err(msg: str) -> None:
    print(msg, file=stderr)


def load_config_or_exit() -> Config:
    if not path.exists(CONFIG_FILE_PATH):
        _err(f"Config file not found. Please create it at '{CONFIG_FILE_PATH}'.")
        exit(1)
    with open(CONFIG_FILE_PATH, "r") as f:
        data = json.load(f)
        try:
            config = Config(**data)
            return config
        except ValidationError as err:
            _err(f"Config file is not valid. File: '{CONFIG_FILE_PATH}'")
            for error in err.errors():
                msg = error["msg"]
                field = ".".join(str(error["loc"]))
                print(f"  {field}: {msg}")
            exit(1)



import re
from typing import List, Optional
import psutil


def is_profile_active(names: List[str], pattern: str, exclude: Optional[str]) -> bool:
    for proc in psutil.process_iter():
        if proc.name() in names:
            cmdline = proc.cmdline()

            # Exclude pattern
            if exclude:
                for cmd in cmdline:
                    if re.search(exclude, cmd):
                        continue

            for cmd in cmdline:
                if re.search(pattern, cmd):
                    return True
    return False


if __name__ == "__main__":
    print(is_profile_active(["chrome"], r"--user-data-dir.*", r"--type=renderer"))
